@@ -1,19 +1,13 @@
 function osxmakeitoctave3(mode)
 % This is the MacOSX version of makeit: It is meant for building PTB for
-% 64-Bit Octave-6.4 on macOS 10.15.7 Catalina or later, using the 10.15 SDK.
+% 64-Bit Octave-8.1 on macOS 12.6 Monterey or later, using the 12.x SDK.
 %
-% This build routine also encodes octave runtime library paths for octave
-% version 6.4 into all mex files. The point of this is to make sure mex files
-% don't fail to load due to libraries like liboctinterp.8.dylib not being
-% found. This is a stop-gap measure: It would be much better to modify
-% octave upstream to encode a -rpath = $OCTAVE_LIBS in the octave
-% executable and libraries themselves, and to add a
-% @rpath/liboctinterp.8.dylib etc. to the mkoctfile build function.
-%
-% This rpath encoding is done by the helper function osxsetoctaverpath().
+% The rpath Octave version independent encoding is done by the helper
+% function osxsetoctaverpath().
+dst = [PsychtoolboxRoot 'PsychBasic/Octave8OSXFiles64/'];
 
 if ~IsOSX(1) || ~IsOctave
-    error('osxmakeitoctave3 only works with a 64-Bit version of Octave for OSX!');
+    error('osxmakeitoctave3 only works with a 64-Bit version of Octave 8 for macOS!');
 end
 
 if nargin < 1 || isempty(mode)
@@ -72,6 +66,7 @@ if mode==0
     -weak_library,/Library/Frameworks/GStreamer.framework/Versions/Current/lib/libgthread-2.0.dylib,\
     -weak_library,/Library/Frameworks/GStreamer.framework/Versions/Current/lib/libglib-2.0.dylib,\
     -weak_library,/usr/local/lib/libdc1394.dylib,\
+    -rpath,/Library/Frameworks/GStreamer.framework/Versions/Current/lib,\
     -framework,CoreServices,-framework,CoreFoundation,-framework,ApplicationServices,-framework,CoreAudio,-framework,OpenGL,-framework,CoreVideo,\
     -framework,IOKit,-framework,SystemConfiguration,-framework,Carbon,-framework,CoreText,-framework,QuartzCore,\
     -framework,CoreDisplay,-F/System/Library/PrivateFrameworks,-framework,DisplayServices,\
@@ -91,21 +86,21 @@ if mode==0
     movefile('./OSX/Screen/PsychCocoaGlue.m', './OSX/Screen/PsychCocoaGlue.c');
 
     osxsetoctaverpath('Screen');
-    unix(['mv ../Projects/MacOSX/build/Screen.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/Screen.' mexext ' ' dst]);
 end
 
 if mode==1
     % Build GetSecs:
     mex --output ../Projects/MacOSX/build/GetSecs -DPTBMODULE_GetSecs -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include -ICommon/Base -IOSX/Base -ICommon/GetSecs "OSX/Base/*.c" "Common/Base/*.c" "Common/GetSecs/*.c"
     osxsetoctaverpath('GetSecs');
-    unix(['mv ../Projects/MacOSX/build/GetSecs.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/GetSecs.' mexext ' ' dst]);
 end
 
 if mode==2
     % Build WaitSecs:
     mex --output ../Projects/MacOSX/build/WaitSecs -DPTBMODULE_WaitSecs -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include -ICommon/Base -IOSX/Base -ICommon/WaitSecs  "OSX/Base/*.c" "Common/Base/*.c" "Common/WaitSecs/*.c"
     osxsetoctaverpath('WaitSecs');
-    unix(['mv ../Projects/MacOSX/build/WaitSecs.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/WaitSecs.' mexext ' ' dst]);
 end
 
 if mode==3
@@ -113,7 +108,7 @@ if mode==3
     % Build PsychPortAudio:
     mex --output ../Projects/MacOSX/build/PsychPortAudio -DPTBMODULE_PsychPortAudio -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio,-framework,AudioToolbox,-framework,AudioUnit" -I/usr/include -ICommon/Base -IOSX/Base -ICommon/PsychPortAudio  "OSX/Base/*.c" "Common/Base/*.c" "Common/PsychPortAudio/*.c" ../Cohorts/PortAudio/libportaudio_osx_64.a
     osxsetoctaverpath('PsychPortAudio');
-    unix(['mv ../Projects/MacOSX/build/PsychPortAudio.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/PsychPortAudio.' mexext ' ' dst]);
 end
 
 if mode==4
@@ -122,14 +117,14 @@ if mode==4
     % Build Eyelink:
     mex --output ../Projects/MacOSX/build/Eyelink -DPTBMODULE_Eyelink -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio,-F/Library/Frameworks/,-framework,eyelink_core" -I/usr/local/include -ICommon/Base -IOSX/Base -ICommon/Eyelink  "OSX/Base/*.c" "Common/Base/*.c" "Common/Eyelink/*.c"
     osxsetoctaverpath('Eyelink');
-    unix(['mv ../Projects/MacOSX/build/Eyelink.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/Eyelink.' mexext ' ' dst]);
 end
 
 if mode==5
     % Build IOPort:
     mex --output ../Projects/MacOSX/build/IOPort -DPTBMODULE_IOPort -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include  -ICommon/Base -IOSX/Base -ICommon/IOPort  "OSX/Base/*.c" "Common/Base/*.c" "Common/IOPort/*.c"
     osxsetoctaverpath('IOPort');
-    unix(['mv ../Projects/MacOSX/build/IOPort.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/IOPort.' mexext ' ' dst]);
 end
 
 if mode==6
@@ -141,7 +136,7 @@ if mode==6
     catch %#ok<*CTCH>
     end
     osxsetoctaverpath('moglcore', './');
-    unix(['mv moglcore.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv moglcore.' mexext ' ' dst]);
     cd(curdir);
 end
 
@@ -150,15 +145,15 @@ if mode==7
     % Build PsychKinectCore:
     mex --output ../Projects/MacOSX/build/PsychKinectCore -DPTBMODULE_PsychKinectCore -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include -I/usr/local/include/libusb-1.0 -I/usr/local/include/libfreenect -ICommon/Base -IOSX/Base -ICommon/PsychKinect  "OSX/Base/*.c" "Common/Base/*.c" "Common/PsychKinect/*.c" -L/usr/local/lib -lfreenect -lusb-1.0
     osxsetoctaverpath('PsychKinectCore');
-    unix(['mv ../Projects/MacOSX/build/PsychKinectCore.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/PsychKinectCore.' mexext ' ' dst]);
 end
 
 if mode==8
     % Build PsychHID:
     % Dynamic link:   mex --output ../Projects/MacOSX/build/PsychHID -DPTBMODULE_PsychHID -DPTBOCTAVE3MEX "-Wl,-headerpad_max_install_names,-framework,ApplicationServices,-framework,Carbon,-framework,CoreAudio,-framework,IOKit" -I/usr/include -ICommon/Base -IOSX/Base -ICommon/PsychHID -IOSX/PsychHID -I../Cohorts/HID_Utilities_64Bit/ -I../Cohorts/HID_Utilities_64Bit/IOHIDManager  "Common/PsychHID/*.c" "OSX/PsychHID/*.c" "OSX/Base/*.c" "Common/Base/*.c" -L../Cohorts/HID_Utilities_64Bit/build/Release -lHID_Utilities
-    mex --output ../Projects/MacOSX/build/PsychHID -DPTBMODULE_PsychHID -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,ApplicationServices,-framework,CoreServices,-framework,CoreFoundation,-framework,Carbon,-framework,CoreAudio,-framework,IOKit" -I/usr/include -ICommon/Base -IOSX/Base -ICommon/PsychHID -IOSX/PsychHID -I../Cohorts/HID_Utilities_64Bit/ -I../Cohorts/HID_Utilities_64Bit/IOHIDManager  "Common/PsychHID/*.c" "OSX/PsychHID/*.c" "OSX/Base/*.c" "Common/Base/*.c" ../Cohorts/HID_Utilities_64Bit/build/Release/libHID_Utilities64.a
+    mex --output ../Projects/MacOSX/build/PsychHID -DPTBMODULE_PsychHID -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,ApplicationServices,-framework,CoreServices,-framework,CoreFoundation,-framework,Carbon,-framework,CoreAudio,-framework,IOKit,-weak_library,/usr/local/lib/libusb-1.0.dylib" -I/usr/local/include/libusb-1.0 -ICommon/Base -IOSX/Base -ICommon/PsychHID -IOSX/PsychHID -I../Cohorts/HID_Utilities_64Bit/ -I../Cohorts/HID_Utilities_64Bit/IOHIDManager  "Common/PsychHID/*.c" "OSX/PsychHID/*.c" "OSX/Base/*.c" "Common/Base/*.c" ../Cohorts/HID_Utilities_64Bit/build/Release/libHID_Utilities64.a
     osxsetoctaverpath('PsychHID');
-    unix(['mv ../Projects/MacOSX/build/PsychHID.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/PsychHID.' mexext ' ' dst]);
 end
 
 if mode==9
@@ -170,7 +165,7 @@ if mode==9
     catch
     end
     osxsetoctaverpath('moalcore', './');
-    unix(['mv ./moalcore.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ./moalcore.' mexext ' ' dst]);
 
     cd(curdir);
 end
@@ -179,7 +174,7 @@ if mode==10
     % Build FontInfo:
     mex --output ../Projects/MacOSX/build/FontInfo -DPTBMODULE_FontInfo -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,ApplicationServices,-framework,CoreAudio" -I/usr/include -ICommon/Base -ICommon/Fonts -IOSX/Base -IOSX/Fonts  "Common/Base/*.c" "OSX/Base/*.c" "OSX/Fonts/*.c"
     osxsetoctaverpath('FontInfo');
-    unix(['mv ../Projects/MacOSX/build/FontInfo.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/FontInfo.' mexext ' ' dst]);
 end
 
 if mode==11
@@ -190,24 +185,24 @@ if mode==11
     % MachAbsoluteTimeClockFrequency
     mex --output ../Projects/MacOSX/build/MachAbsoluteTimeClockFrequency -DPTBMODULE_MachAbsoluteTimeClockFrequency -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include -ICommon/Base -IOSX/Base -ICommon/MachAbsoluteTimeClockFrequency  "OSX/Base/*.c" "Common/Base/*.c" "Common/MachAbsoluteTimeClockFrequency/*.c"
     osxsetoctaverpath('MachAbsoluteTimeClockFrequency');
-    unix(['mv ../Projects/MacOSX/build/MachAbsoluteTimeClockFrequency.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/MachAbsoluteTimeClockFrequency.' mexext ' ' dst]);
 
     % MachGetPriorityMex
     mex --output ../Projects/MacOSX/build/MachGetPriorityMex -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include -ICommon/MachPriorityMex "Common/MachPriorityMex/MachPriorityCommonMex.c" "Common/MachPriorityMex/MachGetPriorityMex.c"
     osxsetoctaverpath('MachGetPriorityMex');
-    unix(['mv ../Projects/MacOSX/build/MachGetPriorityMex.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/MachGetPriorityMex.' mexext ' ' dst]);
 
     % MachSetPriorityMex
     mex --output ../Projects/MacOSX/build/MachSetPriorityMex -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include -ICommon/MachPriorityMex "Common/MachPriorityMex/MachPriorityCommonMex.c" "Common/MachPriorityMex/MachSetPriorityMex.c"
     osxsetoctaverpath('MachSetPriorityMex');
-    unix(['mv ../Projects/MacOSX/build/MachSetPriorityMex.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/MachSetPriorityMex.' mexext ' ' dst]);
 end
 
 if mode==12
     % Build Gestalt:
     mex --output ../Projects/MacOSX/build/Gestalt -DPTBMODULE_Gestalt -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio" -I/usr/include -ICommon/Base -IOSX/Base -IOSX/Gestalt -IOSX/OS9ToolboxFragments  "OSX/Base/*.c" "Common/Base/*.c" "OSX/Gestalt/*.c"
     osxsetoctaverpath('Gestalt');
-    unix(['mv ../Projects/MacOSX/build/Gestalt.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/Gestalt.' mexext ' ' dst]);
 end
 
 if mode==13
@@ -217,15 +212,16 @@ if mode==13
     try
         mex --output ./pnet pnet.c "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names"
         osxsetoctaverpath('pnet', './');
-        unix(['mv ./pnet.' mexext ' ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+        unix(['mv ./pnet.' mexext ' ' dst]);
     catch
     end
     cd(curdir);
 end
 
-if mode==14
+if mode==14 && false
     % Build PsychOculusVRCore.mex:
-    % Depends on Oculus VR SDK v0.5
+    % Depends on Oculus VR SDK v0.5 - Does not work on macOS 10.15 or later
+    % due to the VR runtime being unsupported 32-Bit Intel only.
     try
         mex --output ../Projects/MacOSX/build/PsychOculusVRCore -DPTBMODULE_PsychOculusVRCore -DPTBOCTAVE3MEX "-Wno-deprecated-declarations -mmacosx-version-min='10.11'" "-Wl,-headerpad_max_install_names,-F/Library/Frameworks/,-framework,CoreServices,-framework,CoreFoundation,-framework,CoreAudio,-framework,LibOVR" -ICommon/Base -IOSX/Base -ICommon/PsychOculusVRCore -I/Library/Frameworks/LibOVR.framework/Versions/Current/Headers/ OSX/Base/*.c Common/Base/*.c Common/PsychOculusVRCore/*.c
     catch
@@ -233,7 +229,7 @@ if mode==14
     end
 
     osxsetoctaverpath('PsychOculusVRCore');
-    unix(['mv ../Projects/MacOSX/build/PsychOculusVRCore.mex ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/PsychOculusVRCore.mex ' dst]);
 end
 
 if mode==15
@@ -249,7 +245,7 @@ if mode==15
     end
 
     osxsetoctaverpath('PsychVulkanCore');
-    unix(['mv ../Projects/MacOSX/build/PsychVulkanCore.mex ' PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles64/']);
+    unix(['mv ../Projects/MacOSX/build/PsychVulkanCore.mex ' dst]);
 end
 
 return;
@@ -280,7 +276,7 @@ function mex(varargin)
       outargs = {outargs{:}, filtered};
     end
   end
-  
+
   args = cellstr(char(outargs));
   mkoctfile (args{:});
 end
