@@ -42,6 +42,7 @@ function callStack = AssertMex(varargin)
 %                   and Matlab < V7.4 (aka R2007a) no longer supported.
 % 23-Feb-2019 mk    Make more robust in case of wrong path order.
 % 07-Aug-2019 mk    Update for new MSVC 2019 build system on Windows-10 / Win 10 SDK.
+% 13-Feb-2023 mk    Update for PTB 3.0.19.
 
 persistent okNames mexExtensions;
 
@@ -51,13 +52,13 @@ if IsOctave
     if isempty(strfind(myName, '.m'))
         return;
     end
-    
+
     octFilename = [ myName(1:end-1) 'mex'];
     fprintf('\nIn place of the expected Octave .mex binary plugin file this placeholder file was executed:\n\n');
     fprintf(['  ' myName '\n\n']);
     fprintf('This MEX file seems to be missing or inaccessible on your Octave path or it is dysfunctional:\n\n')
     fprintf(['  ' octFilename '\n\n']);
-    
+
     fpath = which(myName(1:end-2));
     if isempty(fpath)
         fprintf('Hmm. I cannot find the file on your Octave path?!?\n\n');
@@ -76,21 +77,27 @@ if IsOctave
             else
                 oext = filesep;
             end
-            
+
             if IsLinux
-                %fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave3LinuxFiles' oext]);
+                if ~IsARM
+                    % 64-Bit Intel:
+                    fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave5LinuxFiles' oext]);
+                else
+                    % 32-Bit ARM:
+                    fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave3LinuxFilesARM' oext]);
+                end
             end
             if IsOSX
-                fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles' oext]);
+                fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave8OSXFiles' oext]);
             end
             if IsWindows
-                fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic\Octave6WindowsFiles' oext]);
+                fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic\Octave8WindowsFiles' oext]);
             end
         else
             % Correct file with correct extension, still load failure:
             % Check for supported Octave version:
             fprintf('Your version of Octave (%s) might be incompatible with Psychtoolbox: We support Octave 5.0.0 or later\n', version);
-            fprintf('on Linux, Octave 6.4 on Windows and macOS.\n');
+            fprintf('on Linux, Octave 7.3 on Windows and Octave 7.3/8.1 on macOS.\n');
 
             fprintf('Another reason could be some missing 3rd party dynamic link shared libraries on your system.\n');
             fprintf('Another reason could be some binary incompatibility. You would need to recompile Psychtoolbox from source!\n\n');
@@ -118,13 +125,13 @@ if isempty(inputNames) || ismember(computer, inputNames)
     % Element 1 will always be AssertMex. Element 2 will be the calling
     % function unless it is invoked from the commnand line.
     callStack = dbstack;
-    
+
     if length(callStack) > 1
         callerName = callStack(2).name;
     else
         error('AssertMex was invoked from the command line.');
     end
-    
+
     % Generate error strings
     extensionNameIndex=find(streq(computer,okNames));
     extensionName=mexExtensions{extensionNameIndex}; %#ok<FNDSB>
@@ -140,7 +147,7 @@ if isempty(inputNames) || ismember(computer, inputNames)
     fprintf(['  ' callerName '\n\n']);
     fprintf('This mex file seems to be missing or inaccessible on your Matlab path or it is dysfunctional:\n\n')
     fprintf(['  ' mexFilename '\n\n']);
-    
+
     if isempty(which(mexFilename))
         if strcmp(computer, 'MAC')
             % Mac PowerPC:
